/*
 * Common image-related functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "display.h"
#include "image.h"



/* Obtain the position for the stored region from the given frame. */

position_t *image_get_stored_position(stored_regions_t *r, int frame)
{
    return &r->pos[frame];
}

/* Obtain the image data for the stored region from the given frame. */

uint8_t *image_get_stored_region(stored_regions_t *r, int frame)
{
    return r->image + r->size * frame;
}



/* Copy a region from the screen to the store, then blit the image. */

void image_plot_sprite(sprite_t *s, int x, int y, int key)
{
    int frame = s->cfg->frame;
    position_t *p = image_get_stored_position(s->regions, frame);

    /* Copy to the stored region. */

    display_copy(s->cfg, image_get_stored_region(s->regions, frame),
                 s->image->width, s->image->height / s->yscale, 1,
                 x, y, -1, 0);

    /* Plot to the screen. */

    display_copy(s->cfg, s->image->image,
                 s->image->width, s->image->height, s->yscale,
                 x, y, key, 1);

    /* Record the stored background details. */

    p->x = x;
    p->y = y;

    if (frame >= s->regions->stored)
        s->regions->stored = frame + 1;
}

/* Copy a region from the store to the screen, restoring the original
   background. */

void image_unplot_sprite(sprite_t *s)
{
    int frame = s->cfg->frame;
    position_t *p = image_get_stored_position(s->regions, frame);

    /* Only unplot the sprite if a region was stored for the frame. */

    if (s->regions->stored > frame)
        display_copy(s->cfg, image_get_stored_region(s->regions, frame),
                     s->image->width, s->image->height / s->yscale, 1,
                     p->x, p->y, -1, 1);
}
