/*
 * Some simple utility functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "utils.h"



/* A simple busy wait involving a register-based counter. */

void wait(uint32_t delay)
{
    uint32_t counter = delay;

    if (!delay) return;
    while (counter--) __asm__("");      /* retain loop */
}

/* Wrap a pointer within the given limits. */

uint8_t *wrap_pointer(uint8_t *ptr, uint8_t *lower, uint8_t *upper)
{
    uint32_t size = upper - lower;

    if (ptr < lower)
        return upper - (lower - ptr) % size;
    else if (ptr >= upper)
        return lower + (ptr - upper) % size;
    else
        return ptr;
}

/* Wrap a value within the bounds [0, limit). */

int wrap_value(int value, int limit)
{
    if (value < 0)
        return limit - (-value % limit);
    else if (value >= limit)
        return (value - limit) % limit;
    else
        return value;
}
